
// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW  : WORLD;        //the models world matrix
float4x4 tV  : VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tWV : WORLDVIEW;
float4x4 tVP : VIEWPROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;
float4x4 tP  : PROJECTION;   //projection matrix as set via Renderer (EX9)

//light properties
float3 lDir <string uiname="Light Direction";> = {0, -5, 2};        //light direction in world space
float4 lAmb  : COLOR <String uiname="Ambient Color";>  = {0.15, 0.15, 0.15, 1};
float4 lDiff : COLOR <String uiname="Diffuse Color";>  = {0.85, 0.85, 0.85, 1};
float4 lSpec : COLOR <String uiname="Specular Color";> = {0.35, 0.35, 0.35, 1};
float lPower <String uiname="Power"; float uimin=0.0;> = 25.0;     //shininess of specular highlight

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

float4x4 TT ;
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;
float4x4 tColor <string uiname="Color Transform";>;

struct vs2ps
{
    float4 PosWVP    : POSITION;
    float4 TexCd     : TEXCOORD0;
    float3 LightDirV : TEXCOORD1;
    float2 UV        : TEXCOORD2;
    float3 ViewDirV  : TEXCOORD3;
    float3 Pos       : TEXCOORD4;
};

#define TWOPI 6.28318531
#define PI    3.14159265

// --------------------------------------------------------------------------------------------------
// FUNCTIONS:
// --------------------------------------------------------------------------------------------------


float gridSpaceX;
float gridSpaceY;

float gridOffsetX;
float gridOffsetY;

float gridScaleX = 1;
float gridScaleY = 1;

float param1 = 1;
float param2 = 1;
float param3 = 1;
float param4 = 1;


////////////////////////////////////////////////////////////////////////////////
/*Paraboloid
x= a (u/H)^0,5 cos(v)
y= a (u/H)^0,5 sin(v)
z= u
0<u<infinty,0<v<2pi
*///////////////////////////////////////////////////////////////////////////////

float3 Paraboloid(float u, float v)
{
       float x = param1 * pow((u/param2),0.5) * cos(v);
       float y = param1 * pow((u/param2),0.5) * sin(v);
       float z = (u);

      return float3(x, y, z);
}



////////////////////////////////////////////////////////////////////////////////
/*Hyperboloid
x= a (1+u2)^0,5 cos(v)
y= b (1+u2)^0,5 sin(v)
z= c (u)
0<u< 2pi, 0<v<2pi
*///////////////////////////////////////////////////////////////////////////////

float3 Hyperboloid(float u, float v)
{
       float x = param1 * pow((1 + pow(u,2)),0.5) * cos(v);
       float y = param2 * pow((1 + pow(u,2)),0.5) * sin(v);
       float z = param3 * (u);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Eight Surface
x= cos(u) sin(2v)
y= sin (u) sin(2v)
z= sin (v)
0<u< 2pi, -pi/2<v<pi/2
*///////////////////////////////////////////////////////////////////////////////

float3 Eight(float u, float v)
{
       float x = cos(u) * sin(2*v);
       float y = sin(u) * sin(2*v);
       float z = sin(v);

      return float3(x, y, z);
}



////////////////////////////////////////////////////////////////////////////////
/*Folium/////
x = cos(u) (2 v/ pi - tanh(v))
y = cos(u + 2 pi / 3) / cosh(v)
z = cos(u - 2 pi / 3) / cosh(v)
-pi <= u <= pi, -pi <= v <= pi
*///////////////////////////////////////////////////////////////////////////////

float3 Folium(float u, float v)
{
       float x = cos(u) * (2 * v/ PI - tanh(v));
       float y = cos(u + TWOPI / 3) / cosh(v);
       float z = cos(u - TWOPI / 3) / cosh(v);

      return float3(x, y, z);
}



////////////////////////////////////////////////////////////////////////////////
/*Limpet Torus/////
x = cos(u) / [sqrt(2) + sin(v)]
y = sin(u) / [sqrt(2) + sin(v)]
z = 1 / [sqrt(2) + cos(v)]
*///////////////////////////////////////////////////////////////////////////////

float3 LimpetTorus(float u, float v)
{
       float x = cos(u) / (sqrt(2) + sin(v));
       float y = sin(u) / (sqrt(2) + sin(v));
       float z = 1 / (sqrt(2) + cos(v));

      return float3(x, y, z);
}


////////////////////////////////////////////////////////////////////////////////
/*Cresent/////
x = (2 + sin(2 pi u) sin(2 pi v)) sin(3 pi v)
y = (2 + sin(2 pi u) sin(2 pi v)) cos(3 pi v)
z = cos(2 pi u) sin(2 pi v) + 4 v - 2
0 <= u <= 1, 0 <= v <= 1
*///////////////////////////////////////////////////////////////////////////////

float3 Cresent(float u, float v)
{

       float x = (2 + sin(TWOPI * u) * sin(TWOPI * v)) * sin(3 * PI * v);
       float y = (2 + sin(TWOPI * u) * sin(TWOPI * v)) * cos(3 * PI * v);
       float z = cos(TWOPI * u) * sin(TWOPI * v) + 4 * v - 2;

      return float3(x, y, z);
}



////////////////////////////////////////////////////////////////////////////////
/*Figure8Torus/////
x = cos(u) (c + sin(v) cos(u) - sin(2 v) sin(u) / 2)
y = sin(u) (c + sin(v) cos(u) - sin(2 v) sin(u) / 2)
z = sin(u) sin(v) + cos(u) sin(2 v) / 2
-pi <= u,v <= pi
*///////////////////////////////////////////////////////////////////////////////

float3 Figure8Torus(float u, float v)
{

    float x = cos(u) * (param3 + sin(v) * cos(u) - sin(2*v) * sin(u) / 2);
    float y = sin(u) * (param3 + sin(v) * cos(u) - sin(2*v) * sin(u) / 2) ;
    float z = sin(u) * sin(v) + cos(u) * sin(2*v) / 2;

    return float3(x, y, z);
}





////////////////////////////////////////////////////////////////////////////////
/*Triaxial Teardrop/////
x = [1 - cos(u)] cos(u + 2 pi / 3) cos(v + 2 pi / 3) / 2;
y = [1 - cos(u)] cos(u + 2 pi /3) cos(v - 2 pi / 3) / 2;
z = cos(u - 2 pi / 3);
0 <= u <= pi, 0 <= v <= 2 pi
*///////////////////////////////////////////////////////////////////////////////

float3 TriaxialTeardrop(float u, float v)
{

    float x = (1 - cos(u)) * cos(u + TWOPI / 3) * cos(v + TWOPI / 3) / 2;
    float y = (1 - cosh(u)) * cos(u + TWOPI /3) * cos(v - TWOPI / 3) / 2;
    float z = cos(u - TWOPI / 3) ;

    return float3(x, y, z);
}



////////////////////////////////////////////////////////////////////////////////
/*Bent Horns/////
x = (2 + cos(u)) (v/3 - sin(v))
y = (2 + cos(u - 2 PI / 3)) (cos(v) - 1)
z = (2 + cos(u + 2 PI / 3)) (cos(v) - 1)
-pi <= u <= pi,-2pi <= v <= 2pi
*///////////////////////////////////////////////////////////////////////////////

float3 BentHorns(float u, float v)
{

    float x = (2 + cos(u)) * (v/3 - sin(v));
    float y = (2 + cos(u - TWOPI/3)) * (cos(v) - 1);
    float z = (2 + cos(u + TWOPI/3)) * (cos(v) - 1);

    return float3(x, y, z);
}



////////////////////////////////////////////////////////////////////////////////
/*Pillow shape/////
x = cos(u)
y = cos(v)
z = sin(u) sin(v)
-pi <= u <= pi, -pi <= v <= pi
*///////////////////////////////////////////////////////////////////////////////



// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Paraboloid(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= 1;
    gridScaleY *= TWOPI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Paraboloid(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}



////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Hyperboloid(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= TWOPI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Hyperboloid(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Eight(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= PI/2;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Eight(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}





////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Folium(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= PI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Folium(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}



////////////////////////////////////////////////////////////////////////////////

vs2ps VS_LimpetTorus(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= PI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =LimpetTorus(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}




////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Cresent(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= 1;
    gridScaleY *= 1;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Cresent(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}



////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Figure8Torus(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= -PI;
    gridScaleY *= PI ;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Figure8Torus(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}



//////////////////////////////////////////////////////////////////////

vs2ps VS_TriaxialTeardrop(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= TWOPI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =TriaxialTeardrop(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}



//////////////////////////////////////////////////////////////////////

vs2ps VS_BentHorns(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= TWOPI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz = BentHorns(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}





// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------


///////////////////////////////////////////////////////////////////////////////////

float4 PS_Paraboloid(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Paraboloid(u1, In.UV.y) - Paraboloid(u2, In.UV.y));
    bitang = (Paraboloid(In.UV.x, v1) - Paraboloid(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}



///////////////////////////////////////////////////////////////////////////////////

float4 PS_Hyperboloid(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Hyperboloid(u1, In.UV.y) - Hyperboloid(u2, In.UV.y));
    bitang = (Hyperboloid(In.UV.x, v1) - Hyperboloid(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Eight(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Eight(u1, In.UV.y) - Eight(u2, In.UV.y));
    bitang = (Eight(In.UV.x, v1) - Eight(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}




///////////////////////////////////////////////////////////////////////////////////

float4 PS_Folium(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Folium(u1, In.UV.y) - Folium(u2, In.UV.y));
    bitang = (Folium(In.UV.x, v1) - Folium(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}


///////////////////////////////////////////////////////////////////////////////////

float4 PS_LimpetTorus(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (LimpetTorus(u1, In.UV.y) - LimpetTorus(u2, In.UV.y));
    bitang = (LimpetTorus(In.UV.x, v1) - LimpetTorus(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}




///////////////////////////////////////////////////////////////////////////////////

float4 PS_Cresent(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Cresent(u1, In.UV.y) - Cresent(u2, In.UV.y));
    bitang = (Cresent(In.UV.x, v1) - Cresent(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////





float4 PS_Figure8Torus(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Figure8Torus(u1, In.UV.y) - Figure8Torus(u2, In.UV.y));
    bitang = (Figure8Torus(In.UV.x, v1) - Figure8Torus(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}



///////////////////////////////////////////////////////////////////////////////////

float4 PS_TriaxialTeardrop(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (TriaxialTeardrop(u1, In.UV.y) - TriaxialTeardrop(u2, In.UV.y));
    bitang = (TriaxialTeardrop(In.UV.x, v1) - TriaxialTeardrop(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}



///////////////////////////////////////////////////////////////////////////////////

float4 PS_BentHorns(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (BentHorns(u1, In.UV.y) - BentHorns(u2, In.UV.y));
    bitang = (BentHorns(In.UV.x, v1) - BentHorns(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}





// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------


technique Paraboloid
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Paraboloid();
        PixelShader = compile ps_2_a PS_Paraboloid();
    }
}



technique Hyperboloid
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Hyperboloid();
        PixelShader = compile ps_2_a PS_Hyperboloid();
    }
}

technique Eight
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Eight();
        PixelShader = compile ps_2_a PS_Eight();
    }
}

technique Folium
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Folium();
        PixelShader = compile ps_2_a PS_Folium();
    }
}

technique LimpetTorus
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_LimpetTorus();
        PixelShader = compile ps_2_a PS_LimpetTorus();
    }
}

technique Cresent
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Cresent();
        PixelShader = compile ps_2_a PS_Cresent();
    }
}

technique Figure8Torus
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Figure8Torus();
        PixelShader = compile ps_2_a PS_Figure8Torus();
    }
}



technique TriaxialTeardrop
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_TriaxialTeardrop();
        PixelShader = compile ps_2_a PS_TriaxialTeardrop();
    }
}



technique BentHorns
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_BentHorns();
        PixelShader = compile ps_2_a PS_BentHorns();
    }
}

technique TFallbackGouraudDirectionalFF
{
    pass P0
    {
        //transformations
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        //material
        MaterialAmbient  = {1, 1, 1, 1};
        MaterialDiffuse  = {1, 1, 1, 1};
        MaterialSpecular = {1, 1, 1, 1};
        MaterialPower    = (lPower);

        //texturing
        Sampler[0] = (Samp);
        TextureTransform[0] = (tTex);
        TexCoordIndex[0] = 0;
        TextureTransformFlags[0] = COUNT2;
        //Wrap0 = U;  // useful when mesh is round like a sphere

        //lighting
        LightEnable[0] = TRUE;
        Lighting       = TRUE;
        SpecularEnable = TRUE;

        LightType[0]     = DIRECTIONAL;
        LightAmbient[0]  = (lAmb);
        LightDiffuse[0]  = (lDiff);
        LightSpecular[0] = (lSpec);
        LightDirection[0] = (lDir);

        //shading
        ShadeMode = GOURAUD;
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}
